unit MGDirectDll;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, MGVirtIntf;

const
 DllEntyPointName = 'MGDirectDllEntry001';

type
  TMGDllEntryPointProc = procedure (IApp : TMGIApplicationServicesIntf; var IDll : TMGIDllServicesIntf);stdcall;

{**********************}

  TMGGetEvent = procedure (sender:Tobject; var Value:variant) of object;
  TMGSetEvent = procedure (sender:Tobject; Value:variant) of object;
  TMGCanCloseEvent = procedure (sender:TObject; var Canclose:boolean) of object;
  TMGEnabledEvent = procedure (sender:TObject; IsEnabled:boolean) of object;
  TMGGetEnabledEvent = procedure (sender:TObject; var IsEnabled:boolean) of object;
  TMGGetUnknownEvent = procedure (sender:Tobject; PropertyName : string; var Value:variant) of object;
  TMGSetUnknownEvent = procedure (sender:Tobject; PropertyName : string; Value:variant) of object;
  TMGNotifycationEvent = procedure (sender:Tobject; Event : string; Value:variant) of object;
  TMGExecuteEvent = procedure (sender:TObject; values : array of const; var result:variant) of object;
  TMGRegisterEvent = procedure (sender:TObject; var CanRegister : boolean) of object;
  TMGUnRegisterEvent = procedure (sender:TObject; var CanUnRegister : boolean) of object;
  TMGAppRegisterEvent = procedure (sender:TObject; ProcName:string) of object;
  TMGAppEnabledEvent = procedure (sender:TObject; ProcName:string; isEnabled:boolean) of object;

{**********************}

  EMGDesigntimeOnly = class(Exception);
  EMGDllRegisterError = class(Exception);
  EMGRegisterError = class(Exception);

{**********************}

  TMGProcServer = class;
  TMGProcClient = class;
  TMGDllService = class;
  TMGAppService = class;
  TMGDllList    = class;
  TMGDllItem    = class;
  TMGPropertyItem = class;
  TMGPropertyItemClass = class of TMGPropertyItem;

{**********************}

  TMGPropertyItems = class(TCollection)
  private
   fOwner : TComponent;
   procedure SetItem(Index: Integer; Value: TMGPropertyItem);
   function  GetItem(Index: Integer): TMGPropertyItem;
  protected
   function  GetOwner: TPersistent;override;
  public
   constructor Create(aOwner : TComponent; ItemClass : TMGPropertyItemClass);
   function  Add: TMGPropertyItem;
  published
   property Items[Index: Integer]: TMGPropertyItem read GetItem write SetItem; default;
  end;

{**********************}

  TMGPropertyItem = class(TCollectionItem)
  private
   fOnGet : TMGGetEvent;
   fOnSet : TMGSetEvent;
   fName  : string;
   procedure SetName(value:string);
  protected
  public
   procedure   AssignTo(Dest: TPersistent);override;
   function  DoOnGet:variant;virtual;
   procedure DoOnSet(value:variant);virtual;
  published
   property Name  : string read fName write SetName;
   property OnGet : TMGGetEvent read fOnGet write fOnGet;
   property OnSet : TMGSetEvent read fOnSet write fOnSet;
  end;

{**********************}

  TMGInterfaceComponent = class (TComponent)
  end;

{**********************}

  TMGProcComponent = class(TMGInterfaceComponent)
  private
   fRegistered,
   fEnabled,
   fAutoReg     : boolean;
   fProcName    : string;
   fOnCreate,
   fOnDestroy,
   fOnAfterRegister,
   fOnAfterUnRegister  : TNotifyEvent;
   fOnBeforeRegister   : TMGRegisterEvent;
   fOnBeforeUnregister : TMGUnRegisterEvent;
   fOnGetEnabled       : TMGGetEnabledEvent;
   procedure SetAutoReg(value:boolean);
   procedure SetProcName(value:string);
   procedure SetEnabled(value:boolean);
   function  GetEnabled:boolean;
  protected
   function  DoGetEnabled:boolean;virtual;
   procedure DoSetEnabled(value:boolean);virtual;
   procedure DoOnDestroy;virtual;
   procedure DoOnCreate;virtual;
   function  DoOnBeforeRegister:boolean;virtual;
   function  DoOnBeforeUnRegister:boolean;virtual;
   procedure DoOnAfterRegister;virtual;
   procedure DoOnAfterUnRegister;virtual;
   procedure DoSetRegistered(value:boolean);virtual;
  public
   constructor Create(aOwner:TComponent);override;
   destructor  destroy;override;
   procedure Loaded;override;
   procedure Register;virtual;abstract;
   procedure UnRegister;virtual;abstract;
   property  Registered : boolean read fRegistered;
  published
   property AutoRegister : boolean read fAutoReg write SetAutoReg;
   property ProcName : string read fProcName write SetProcName;
   property Enabled : boolean read GetEnabled write SetEnabled;
   property OnBeforeRegister : TMGRegisterEvent read fOnBeforeRegister write fOnBeforeRegister;
   property OnAfterRegister : TNotifyEvent read fOnAfterRegister write fOnAfterRegister;
   property OnBeforeUnRegister : TMGUnRegisterEvent read fOnBeforeUnRegister write fOnBeforeUnRegister;
   property OnAfterUnRegister : TNotifyEvent read fOnAfterUnRegister write fOnAfterUnRegister;
   property OnCreate : TNotifyEvent read fOnCreate write fOnCreate;
   property OnDestroy : TNotifyEvent read fOnDestroy write fOnDestroy;
   property OnGetEnabled : TMGGetEnabledEvent read fOnGetEnabled write fOnGetEnabled;
  end;

{**********************}

  TMGIProcServerStd = class (TMGIProcServerIntf)
  private
   fServerComp : TMGProcServer;
  public
   constructor create(aServer : TMGProcServer);
   function  Execute (Values: array of const):variant;override;
   function  GetProcName:String;override;
   function  GetComment:String;override;
   function  GetType:string;override;
   function  GetIcon:Integer;override;
   function  IsVisible:boolean;override;
   function  IsEnabled:boolean;override;
   function  CanClose:boolean;override;
   function  GetProperty(Prop:string):variant;override;
   procedure SetProperty(Prop:string; value:variant);override;
   procedure NotifyClients(Event:string; value:variant);override;
   procedure NotifyEnabled(isEnabled:boolean);override;
   procedure Register;override;
   procedure UnRegister;override;
  end;

{**********************}

  TMGIProcClientStd = class (TMGIProcClientIntf)
  private
   fClientComp : TMGProcClient;
  public
   constructor create(aClient : TMGProcClient);
   function  GetProcName:string;override;
   function  IsServerEnabled:boolean;override;
   procedure OnEnabledChange(IsAvailable:boolean);override;
   procedure OnServerEvent(Event:string; const value:variant);override;
   function  GetEnabled:boolean;override;
   function  Execute(values:array of const):variant;override;
   procedure Register;override;
   procedure UnRegister;override;
  end;

{**********************}

  TMGIDllServiceStd = class(TMGIDllServicesIntf)
  private
   fDllServiceComp : TMGDllService;
  public
   constructor create(aService:TMGDllService);
   procedure RegisterDll;override;
   procedure UnRegisterDll;override;
   function  GetAuthor:string;override;
   function  GetComment:String;override;
   function  GetCaption:String;override;
   function  GetVersion:string;override;
   function  GetIDString:string;override;
   function  GetType:string;override;
   function  GetIcon:Integer;override;
  end;

{**********************}

  TMGIApplicationServicesStd = class(TMGIApplicationServicesIntf)
  private
   fServiceComp : TMGAppService;
  public
   constructor create(aService : TMGAppService);
   function  GetApplicationHInstance:integer;override;
   procedure RegisterProcServer(Proc:TMGIProcServerIntf);override;
   procedure RegisterProcClient(Proc:TMGIProcClientIntf);override;
   procedure UnRegisterProcServer(Proc:string);override;
   procedure UnRegisterProcClient(Proc:TMGIProcClientIntf);override;
   function  ExecuteProc(Proc:string; const Values: array of const):variant;override;
   procedure NotifyClients(Proc,Event:string; value:variant);override;
   procedure NotifyEnabled(Proc:string; value:boolean);override;
   function  GetProc(Proc:string):TMGIProcServerIntf;override;
   function  IsProcEnabled(Proc:string):boolean;override;
  end;

{**********************}

  TMGProcServer = class(TMGProcComponent)
  private
   fIProcServer : TMGIProcServerStd;
   fPropertyCol : TMGPropertyItems;
   fVisible     : boolean;
   fComment,
   fTyp         : string;
   fIcon        : TIcon;
   fOnGetUnknown       : TMGGetUnknownEvent;
   fOnSetUnknown       : TMGSetUnknownEvent;
   fOnExecute          : TMGExecuteEvent;
   fOnCanClose         : TMGCanCloseEvent;
   procedure SetComment(value:string);
   procedure SetVisible(value:boolean);
   procedure SetIcon(value:TIcon);
   procedure SetTyp(value:string);
   // Schnittstellen zu fIProcServer;
  protected
   procedure DoSetProperty(PropertyName:string;value:variant);virtual;
   function  DoGetProperty(PropertyName:string):variant;virtual;
   function  DoGetUnknownProperty(PropertyName:string):variant;virtual;
   procedure DoSetUnknownProperty(PropertyName:string;value:variant);virtual;
   function  DoCanClose:boolean;virtual;
   function  DoExecute(values : array of const):variant;virtual;
   procedure DoSetEnabled(value:boolean);override;
  public
   constructor Create(aOwner:TComponent);override;
   destructor  destroy;override;
   procedure NotifyClients(Event:String; Value:variant);
   procedure NotifyEnabled;
   procedure Register;override;
   procedure UnRegister;override;
  published
   property Comment : String read fComment write SetComment;
   property Typ : string read fTyp write SetTyp;
   property Icon : TIcon read fIcon write SetIcon;
   property Visible : boolean read fVisible write SetVisible;
   property Properties : TMGPropertyItems read fPropertyCol write fPropertyCol;
   property OnCanClose : TMGCanCloseEvent read fOnCanClose write fOnCanClose;
   property OnGetUnknownProperty : TMGGetUnknownEvent read fOnGetUnknown write fOnGetUnknown;
   property OnSetUnknownProperty : TMGSetUnknownEvent read fOnSetUnknown write fOnSetUnknown;
   property OnExecute : TMGExecuteEvent read fOnExecute write fOnExecute;
  end;

{**********************}

  TMGProcClient = class(TMGProcComponent)
  private
   fIProcClient : TMGIProcClientStd;
   fOnNotify    : TMGNotifycationEvent;
   fOnServerEnabled   : TMGEnabledEvent;
   function GetServerEnabled:boolean;
  protected
   procedure DoOnNotification(event:string; value:variant);virtual;
   Procedure DoOnServerEnabled(value:boolean);virtual;
  public
   constructor Create(AOwner:Tcomponent);override;
   destructor  destroy;override;
   function Execute(values:array of const):variant;
   procedure Register;override;
   procedure UnRegister;override;
   property ServerEnabled : boolean read GetServerEnabled;
  published
   property OnNotifyEnabled : TMGEnabledEvent read fOnServerEnabled write fOnServerEnabled;
   property OnNotification : TMGNotifycationEvent read fOnNotify write fOnNotify;
  end;

{**********************}

  TMGAppService = class(TMGInterfaceComponent)
  private
   fIAppServices  : TMGIApplicationServicesStd;
   fDllPaths      : TStrings;
   fOnCreate,
   fOnDestroy     : TNotifyEvent;
   fOnRegServer,
   fOnUnregServer,
   fOnRegClient,
   fOnUnRegClient : TMGAppRegisterEvent;
   fOnProcEnabled : TMGAppEnabledEvent;
   procedure SetDllPaths(value:TStrings);
   function  GetProc(index:variant):TMGIProcServerIntf;
  protected
   function  DoExecuteProc(Proc:string; const Values: array of const):variant;virtual;
   procedure DoRegisterProcServer(Proc:TMGIProcServerIntf);virtual;
   procedure DoRegisterProcClient(Proc:TMGIProcClientIntf);virtual;
   procedure DoUnRegisterProcServer(Proc:string);virtual;
   procedure DoUnRegisterProcClient(Proc:TMGIProcClientIntf);virtual;
   procedure DoNotifyEnabled(Proc:string; value:boolean);virtual;
   function  DoGetProc(Proc:string):TMGIProcServerIntf;virtual;
   function  DoIsProcEnabled(Proc:string):boolean;virtual;
   procedure DoOnCreate;virtual;
   procedure DoOnDestroy;virtual;
   procedure DoOnRegisterServer(proc:string);virtual;
   procedure DoOnRegisterClient(proc:string);virtual;
   procedure DoOnUnRegisterServer(proc:string);virtual;
   procedure DoOnUnRegisterClient(proc:string);virtual;
   procedure DoOnProcEnabled(Procname:string; isEnabled:boolean);virtual;
   procedure DoRegister;virtual;
  public
   fServerList,
   fClientList    : TStringList;
   DllList : TMGDllList;
   constructor Create(aOwner:TComponent);override;
   destructor  destroy;override;
   procedure loaded;override;
   procedure NotifyClients(Proc,Event:string; value:variant);
   function  RegisterDll(DllPath:string):boolean;
   procedure UnRegisterdll(fdll : TMGIDllServicesIntf);
   function  ProcExists(ProcName:string; var Proc:TMGIProcServerIntf):boolean;virtual;
   function  CanCloseQuery:boolean;virtual;
   property Procs[index:variant] : TMGIProcServerIntf read GetProc;
  published
   property DllPaths : TStrings read fDllPaths write SetDllPaths;
   property OnCreate : TNotifyEvent read fOnCreate write fOnCreate;
   property OnDestroy : TNotifyEvent read fOnDestroy write fOnDestroy;
   property OnRegisterServer : TMGAppRegisterEvent read fOnRegServer write fOnRegServer;
   property OnUnRegisterServer : TMGAppRegisterEvent read fOnUnRegServer write fOnUnRegServer;
   property OnRegisterClient : TMGAppRegisterEvent read fOnRegClient write fOnRegClient;
   property OnUnRegisterClient : TMGAppRegisterEvent read fOnUnRegClient write fOnUnRegClient;
   property OnProcEnabledChange : TMGAppEnabledEvent read fOnProcEnabled write fOnProcEnabled;
  end;

{**********************}

  TMGDllService = class(TMGInterfaceComponent)
  private
   fIDllServices : TMGIDllServicesIntf;
   fAuthor,
   fComment,
   fCaption,
   fVersion,
   fID,
   fTyp     : string;
   fIcon    : TIcon;
   fOnBeforeReg,
   fOnAfterReg,
   fOnCreate,
   fOnDestroy  : TNotifyEvent;
   procedure SetAuthor(value:string);
   procedure SetComment(value:string);
   procedure SetCaption(value:string);
   procedure SetVersion(value:string);
   procedure SetID(value:string);
   procedure SetTyp(value:string);
   procedure SetIcon(value:TIcon);
  protected
   procedure loaded;override;
   procedure DoOnCreate;virtual;
   procedure DoOnDestroy;virtual;
   procedure DoOnBeforeRegister;virtual;
   procedure DoOnAfterRegister;virtual;
   procedure DoRegister;virtual;
   procedure DoUnRegister;virtual;
  public
   constructor Create(AOwner:Tcomponent);override;
   destructor  destroy;override;
  published
   property Author : string read fAuthor write SetAuthor;
   property Comment : string read fComment write SetComment;
   property Caption : string read fCaption write SetCaption;
   property Version : string read fVersion write SetVersion;
   property ID : string read fID write SetID;
   property Typ : string read fTyp write SetTyp;
   property Icon : TIcon read fIcon write SetIcon;
   property OnBeforeRegister : TNotifyEvent read fOnBeforeReg write fOnBeforeReg;
   property OnAfterRegister : TNotifyEvent read fOnAfterReg write fOnAfterReg;
   property OnCreate : TNotifyEvent read fOnCreate write fOnCreate;
   property OnDestroy : TNotifyEvent read fOnDestroy write fOnDestroy;
  end;

{**********************}

  TMGDllItem = class(TPersistent)
  private
   fFilename : string;
   fHandle   : THandle;
   fID       : string;
   fDllService : TMGIDllServicesIntf;
  published
   property Filename : string read fFilename;
   property Handle : THandle read fHandle;
   property ID : string read fID;
   property DllService : TMGIDllServicesIntf read fDllService;
  end;

{**********************}

  TMGDllList = class(TObject)
  private
   fList : TStringList;
   function Find(aName:string; var index:integer):boolean;
   function GetDll(index:variant):TMGDllItem;
   function GetCount:integer;
  public
   constructor Create;
   destructor destroy;override;
   procedure Clear;
   procedure Delete(index:integer);
   function Add(aName:string):TMGDllItem;
   function DllExists(index:variant):boolean;
   property Count : integer read GetCount;
   property Dlls[index:variant] : TMGDllItem read GetDll; default;
  end;

{**********************}

procedure RegisterDll(IApp : TMGIApplicationServicesIntf; var IDll : TMGIDllServicesIntf);stdcall;

procedure Register;

{**********************}

var
   MGDirectDllApplication : TMGIApplicationServicesIntf;
   DllService             : TMGIDllServicesIntf;
   MGDirectDllMainForm    : TFormClass;

implementation

var DllMainForm : TForm;
{**********************}

constructor TMGDllList.Create;
begin
 inherited Create;
 fList := TStringList.Create;
end;

destructor TMGDllList.destroy;
begin
 fList.free;
 inherited destroy;
end;

function TMGDllList.Find(aName:string; var index:integer):boolean;
begin
  index := 0;
  while (index<fList.Count) do
  begin
   if aName=TMGDllItem(fList.Objects[index]).ID then break;
   inc(index);
  end;
  if index = fList.Count then result := false else result := true;
end;

function TMGDllList.GetDll(index:variant):TMGDllItem;
var i : integer;
begin
 result := nil;
 if VarType(index)=varString then
 begin
  if find(index,i) then
   result := TMGDllItem(fList.Objects[i])
 end
 else
 try
   if (index<fList.count) then
    result := TMGDllItem(fList.Objects[index]);
 except
  result := nil;
 end;
end;

function TMGDllList.Add(aName:string):TMGDllItem;
begin
 result := TMGDllItem.create;
 result.fID := aName;
 fList.AddObject(aName,result);
end;

function TMGDllList.GetCount:integer;
begin
 result := fList.Count;
end;

procedure TMGDllList.Clear;
begin
 while fList.count>0 do
  fList.Delete(0);
end;

function TMGDllList.DllExists(index:variant):boolean;
var i: integer;
begin
 if VarType(index)=varString then result := find(index,i)
  else result := fList.Count>0;
end;

procedure TMGDllList.Delete(index:integer);
begin
 if index<fList.Count then
 begin
  fList.Objects[index].free;
  fList.Delete(index);
 end;
end;

{**********************}

procedure TMGPropertyItem.AssignTo(Dest: TPersistent);
begin
 if Dest is TMGPropertyItem then
 begin
  TMGPropertyItem(Dest).OnGet := OnGet;
  TMGPropertyItem(Dest).OnSet := OnSet;
  TMGPropertyItem(Dest).Name := Name;
 end
 else inherited AssignTo(Dest);
end;

function  TMGPropertyItem.DoOnGet:variant;
begin
 result := null;
 if assigned(fOnGet) then OnGet(self,result);
end;

procedure TMGPropertyItem.DoOnSet(value:variant);
begin
 if assigned(fOnSet) then OnSet(self,value);
end;

procedure TMGPropertyItem.SetName(value:string);
begin
 fName := uppercase(value);
end;

{**********************}

constructor TMGPropertyItems.Create(aOwner : TComponent; ItemClass : TMGPropertyItemClass);
begin
 inherited Create(ItemCLass);
 fOwner := aOwner;
end;

procedure TMGPropertyItems.SetItem(Index: Integer; Value: TMGPropertyItem);
begin
  Items[Index].Assign(Value);
end;

function  TMGPropertyItems.GetItem(Index: Integer): TMGPropertyItem;
begin
  Result := TMGPropertyItem(inherited Items[Index]);
end;

function  TMGPropertyItems.GetOwner: TPersistent;
begin
 result := fOwner;
end;

function  TMGPropertyItems.Add: TMGPropertyItem;
begin
 result := TMGPropertyItem(inherited Add);
end;

{**********************}

constructor TMGProcComponent.Create(aOwner:TComponent);
begin
 inherited Create(AOwner);
 fRegistered := false;
 fEnabled    := true;
 fAutoReg    := true;
 fProcName   := '';
end;

destructor  TMGProcComponent.destroy;
begin
 DoOnDestroy;
 inherited destroy;
end;

procedure TMGProcComponent.loaded;
begin
 inherited loaded;
 DoOnCreate;
end;

procedure TMGProcComponent.SetAutoReg(value:boolean);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fAutoReg := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGProcComponent.SetProcName(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fProcName := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGProcComponent.SetEnabled(value:boolean);
begin
 if csLoading in ComponentState then fEnabled := value else DoSetEnabled(value);
end;

function  TMGProcComponent.GetEnabled:boolean;
begin
 if csLoading in ComponentState then result := fEnabled else result := DoGetEnabled;
end;

function  TMGProcComponent.DoGetEnabled:boolean;
begin
 result := fEnabled;
 if Assigned(fOnGetEnabled) then OnGetEnabled(self,result);
{ if result <> fEnabled then DoSetEnabled(result);}
fEnabled := result;
end;

procedure TMGProcComponent.DoSetEnabled(value:boolean);
begin
 fEnabled := value;
end;

procedure TMGProcComponent.DoOnDestroy;
begin
 if assigned(fOnDestroy) then OnDestroy(self);
end;

procedure TMGProcComponent.DoOnCreate;
begin
 if assigned(fOnCreate) then OnCreate(self);
end;

function TMGProcComponent.DoOnBeforeRegister:boolean;
begin
 result := true;
 if assigned(fOnBeforeRegister) then OnBeforeRegister(self,result);
end;

function TMGProcComponent.DoOnBeforeUnRegister:boolean;
begin
 result := true;
 if assigned(fOnBeforeRegister) then OnBeforeUnRegister(self,result);
end;

procedure TMGProcComponent.DoOnAfterRegister;
begin
 if assigned(fOnAfterRegister) then OnAfterRegister(self);
end;

procedure TMGProcComponent.DoOnAfterUnRegister;
begin
 if assigned(fOnAfterUnRegister) then OnAfterUnRegister(self);
end;

procedure TMGProcComponent.DoSetRegistered(value:boolean);
begin
 fRegistered := value;
end;

{**********************}

constructor TMGIProcClientStd.create(aClient : TMGProcClient);
begin
 inherited Create;
 fClientComp := aclient;
end;

function  TMGIProcClientStd.GetProcName:string;
begin
 result := fClientComp.ProcName;
end;

function  TMGIProcClientStd.IsServerEnabled:boolean;
begin
 result := MGDirectDllApplication.IsProcEnabled(GetProcName);
end;

procedure TMGIProcClientStd.OnEnabledChange(IsAvailable:boolean);
begin
 fClientComp.DoOnServerEnabled(IsAvailable);
end;

procedure TMGIProcClientStd.OnServerEvent(Event:string; const value:variant);
begin
 fClientComp.DoOnNotification(Event,value);
end;

function  TMGIProcClientStd.GetEnabled:boolean;
begin
 result := fClientComp.Enabled;
end;

function  TMGIProcClientStd.Execute(values:array of const):variant;
begin
 result := MGDirectDllApplication.ExecuteProc(GetProcName,values);
end;

procedure TMGIProcClientStd.Register;
begin
 MGDirectDllApplication.RegisterProcClient(self);
end;

procedure TMGIProcClientStd.UnRegister;
begin
 MGDirectDllApplication.UnRegisterProcClient(self);
end;

{**********************}

constructor TMGIProcServerStd.create(aServer : TMGProcServer);
begin
 inherited create;
 fServerComp := aServer;
end;

function  TMGIProcServerStd.Execute (Values: array of const):variant;
begin
 result := fServerComp.DoExecute(values);
end;

function  TMGIProcServerStd.GetProcName:String;
begin
 result := fServerComp.ProcName;
end;

function  TMGIProcServerStd.GetComment:String;
begin
 result := fServerComp.Comment;
end;

function  TMGIProcServerStd.GetType:string;
begin
 result := fServerComp.Typ;
end;

function  TMGIProcServerStd.GetIcon:Integer;
begin
 result := fServerComp.Icon.Handle;
end;

function  TMGIProcServerStd.IsVisible:boolean;
begin
 result := fServerComp.Visible;
end;

function  TMGIProcServerStd.IsEnabled:boolean;
begin
 result := fServerComp.Enabled;
end;

function  TMGIProcServerStd.CanClose:boolean;
begin
 result := fServerComp.DoCanClose;
end;

function  TMGIProcServerStd.GetProperty(Prop:string):variant;
begin
 result := fServerComp.DoGetProperty(prop);
end;

procedure TMGIProcServerStd.SetProperty(Prop:string; value:variant);
begin
 fServerComp.DoSetProperty(Prop,value);
end;

procedure TMGIProcServerStd.NotifyClients(Event:string; value:variant);
begin
 MGDirectDllApplication.NotifyClients(fServerComp.ProcName,Event,value);
end;

procedure TMGIProcServerStd.NotifyEnabled(isEnabled:boolean);
begin
 MGDirectDllApplication.NotifyEnabled(fServerComp.ProcName,isenabled);
end;

procedure TMGIProcServerStd.Register;
begin
 MGDirectDllApplication.RegisterProcServer(self);
end;

procedure TMGIProcServerStd.UnRegister;
begin
 MGDirectDllApplication.UnRegisterProcServer(self.GetProcName);
end;

{**********************}

constructor TMGIDllServiceStd.create(aService:TMGDllService);
begin
 inherited create;
 fDllServiceComp := aService;
 DllService := self;
end;

procedure TMGIDllServiceStd.RegisterDll;
begin
 fDllServiceComp.DoRegister;
end;

procedure  TMGIDllServiceStd.UnRegisterDll;
begin
 fDllServiceComp.DoUnRegister;
end;

function  TMGIDllServiceStd.GetAuthor:string;
begin
 result := fDllServiceComp.Author;
end;

function  TMGIDllServiceStd.GetComment:String;
begin
 result := fDllServiceComp.Comment;
end;

function  TMGIDllServiceStd.GetCaption:String;
begin
 result := fDllServiceComp.Caption;
end;

function  TMGIDllServiceStd.GetVersion:string;
begin
 result := fDllServiceComp.Version;
end;

function  TMGIDllServiceStd.GetIDString:string;
begin
 result := fDllServiceComp.ID;
end;

function  TMGIDllServiceStd.GetType:string;
begin
 result := fDllServiceComp.Typ;
end;

function  TMGIDllServiceStd.GetIcon:Integer;
begin
 result := fDllServiceComp.Icon.Handle;
end;

{**********************}

constructor TMGIApplicationServicesStd.create(Aservice:TMGAppService);
begin
 inherited Create;
 fServiceComp := aService;
end;

function  TMGIApplicationServicesStd.GetApplicationHInstance:integer;
begin
 result := hInstance;
end;

procedure TMGIApplicationServicesStd.RegisterProcServer(Proc:TMGIProcServerIntf);
begin
 fServiceComp.DoRegisterProcServer(Proc);
end;

procedure TMGIApplicationServicesStd.RegisterProcClient(Proc:TMGIProcClientIntf);
begin
 fServiceComp.DoRegisterprocclient(Proc);
end;

procedure TMGIApplicationServicesStd.UnRegisterProcServer(Proc:string);
begin
 fServiceComp.DoUnRegisterProcServer(Proc);
end;

procedure TMGIApplicationServicesStd.UnRegisterProcClient(Proc:TMGIProcClientIntf);
begin
 fServiceComp.DoUnRegisterProcClient(Proc);
end;

function  TMGIApplicationServicesStd.ExecuteProc(Proc:string; const Values: array of const):variant;
begin
 result := fServiceComp.DoExecuteProc(Proc, values);
end;

procedure TMGIApplicationServicesStd.NotifyClients(Proc,Event:string; value:variant);
begin
 fServiceComp.NotifyClients(Proc, Event, Value);
end;

procedure TMGIApplicationServicesStd.NotifyEnabled(Proc:string; value:boolean);
begin
 fServiceComp.DoNotifyEnabled(Proc, value);
end;

function  TMGIApplicationServicesStd.GetProc(Proc:string):TMGIProcServerIntf;
begin
 result := fServiceComp.DoGetProc(Proc);
end;

function  TMGIApplicationServicesStd.IsProcEnabled(Proc:string):boolean;
begin
 result := fServiceComp.DoIsProcEnabled(Proc);
end;

{**********************}

constructor TMGProcServer.Create(aOwner:TComponent);
begin
 inherited Create(AOwner);
 fVisible    := true;
 fComment    := '';
 fTyp        := '';
 fIcon       := TIcon.Create;
 fPropertyCol := TMGPropertyItems.create(self,TMGPropertyItem);
 fIProcServer := TMGIProcServerStd.Create(self);
end;

destructor  TMGProcServer.destroy;
begin
 UnRegister;
 fPropertyCol.free;
 fIProcServer.free;
 fIcon.free;
 inherited destroy;
end;

procedure TMGProcServer.SetComment(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fComment := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGProcServer.SetTyp(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fTyp := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGProcServer.SetVisible(value:boolean);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fVisible := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGProcServer.SetIcon(value:TIcon);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fIcon.assign(value)
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGProcServer.DoSetEnabled(value:boolean);
begin
 inherited;
 fIProcServer.NotifyEnabled(value);
end;

function  TMGProcServer.DoCanClose:boolean;
begin
 result := true;
 if Assigned(fOnCanClose) then OnCanClose(self,result);
end;

function  TMGProcServer.DoGetProperty(PropertyName:string):variant;
var i : integer;
    d : boolean;
begin
 result := null;
 PropertyName := uppercase(PropertyName);
 d := false;
 for i := 0 to fPropertyCol.Count-1 do
  if fPropertyCol[i].Name=PropertyName then
  begin
   result := fPropertyCol[i].DoOnGet;
   d := true;
   break;
  end;
 if not d and Assigned(fOnGetUnknown) then OnGetUnknownProperty(self,Propertyname,result);
end;

procedure TMGProcServer.DoSetProperty(PropertyName:string;value:variant);
var i : integer;
begin
 PropertyName := uppercase(PropertyName);
 for i := 0 to fPropertyCol.Count-1 do
  if fPropertyCol[i].Name=PropertyName then
  begin
   fPropertyCol[i].DoOnSet(value);
   break;
  end;
 if Assigned(fOnSetUnknown) then OnSetUnknownProperty(self,Propertyname,value);
end;

function  TMGProcServer.DoGetUnknownProperty(PropertyName:string):variant;
begin
 result := null;
 if Assigned(fOnGetUnknown) then OnGetUnknownProperty(self,Propertyname,result);
end;

procedure TMGProcServer.DoSetUnknownProperty(PropertyName:string;value:variant);
begin
 if Assigned(fOnSetUnknown) then OnSetUnknownProperty(self,Propertyname,value);
end;

function  TMGProcServer.DoExecute(values : array of const):variant;
begin
 result := null;
 if Assigned(fOnExecute) then OnExecute(self,values,result);
end;

procedure TMGProcServer.NotifyClients(Event:String; Value:variant);
begin
 fIProcServer.NotifyClients(Event,value);
end;

procedure TMGProcServer.NotifyEnabled;
begin
 fIProcServer.NotifyEnabled(Enabled);
end;

procedure TMGProcServer.Register;
begin
 if DoOnBeforeRegister then
 begin
  DoSetRegistered(true);
  fIProcServer.Register;
  DoOnAfterRegister;
 end;
end;

procedure TMGProcServer.UnRegister;
begin
 if ((csDestroying in Componentstate) or (DoOnBeforeUnRegister)) and Registered then
 begin
  DoSetRegistered(false);
  fIProcServer.UnRegister;
  DoOnAfterUnRegister;
 end;
end;

{**********************}

constructor TMGProcClient.create(aowner:TComponent);
begin
 inherited create(aOwner);
 fIProcClient := TMGIProcClientStd.Create(self);
end;

destructor  TMGProcClient.destroy;
begin
 Unregister;
 fIProcClient.free;
 inherited destroy;
end;

function TMGProcClient.GetServerEnabled:boolean;
begin
 result := fIProcClient.IsServerEnabled;
end;

function TMGProcClient.Execute(values:array of const):variant;
begin
 result := fIProcCLient.Execute(values);
end;

procedure TMGProcClient.DoOnServerEnabled(value:boolean);
begin
 if Assigned(fOnServerEnabled) then OnNotifyEnabled(self,value);
end;

procedure TMGProcClient.DoOnNotification(event:string; value:variant);
begin
 if Assigned(fOnNotify) then OnNotification(self,event,value);
end;

procedure TMGProcClient.Register;
begin
 if DoOnBeforeRegister then
 begin
  DoSetRegistered(true);
  fIProcClient.Register;
  DoOnAfterRegister;
 end;
end;

procedure TMGProcClient.UnRegister;
begin
 if ((csDestroying in Componentstate) or (DoOnBeforeUnRegister)) and Registered then
 begin
  DoSetRegistered(false);
  fIProcClient.UnRegister;
  DoOnAfterUnRegister;
 end;
end;

{**********************}

constructor TMGDllService.Create(AOwner:Tcomponent);
begin
 inherited Create(aOwner);
 fAuthor  := '';
 fComment := '';
 fCaption := '';
 fVersion := '';
 fID   := '';
 fTyp  := '';
 fIcon := TIcon.Create;
 fIDllServices := TMGIDllServiceStd.Create(self);
 DllService := fIDllServices;
end;

destructor  TMGDllService.destroy;
begin
 DoOnDestroy;
 fIDllServices.free;
 fIcon.free;
 inherited destroy;
end;

procedure TMGDllService.Loaded;
begin
 inherited loaded;
 DoOnCreate;
end;

procedure TMGDllService.SetAuthor(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fAuthor := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.SetComment(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fComment := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.SetCaption(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fCaption := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.SetVersion(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fVersion := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.SetID(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fID := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.SetTyp(value:string);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fTyp := value
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.SetIcon(value:TIcon);
begin
 if (csDesigning in ComponentState) or (csLoading in ComponentState) then fIcon.Assign(value)
  else raise EMGDesigntimeOnly.Create('Can only be changed in designtime !');
end;

procedure TMGDllService.DoRegister;
var I,j : integer;
begin
 DoOnBeforeRegister;
 for i := 0 to screen.Formcount-1 do
  for j := 0 to screen.forms[i].Componentcount-1 do
   if (screen.forms[i].components[j] is TMGProcServer) then
   begin
    if (screen.forms[i].components[j] as TMGProcServer).Autoregister then (screen.forms[i].components[j] as TMGProcServer).register;
   end
   else
    if (screen.forms[i].components[j] is TMGProcClient) then
     if (screen.forms[i].components[j] as TMGProcClient).Autoregister then (screen.forms[i].components[j] as TMGProcClient).register;
 DoOnAfterRegister;
end;

procedure TMGDllService.DoUnRegister;
begin
end;

procedure TMGDllService.DoOnCreate;
begin
 if assigned(fOnCreate) then OnCreate(self);
end;

procedure TMGDllService.DoOnDestroy;
begin
 if assigned(fOnDestroy) then OnDestroy(self);
end;

procedure TMGDllService.DoOnBeforeRegister;
begin
 if Assigned(fOnBeforeReg) then OnBeforeRegister(self);
end;

procedure TMGDllService.DoOnAfterRegister;
begin
 if Assigned(fOnAfterReg) then OnAfterRegister(self);end;

{********************}

constructor TMGAppService.Create(aOwner:TComponent);
begin
 inherited Create(aOwner);
 DllList    := TMGDllList.Create;
 fDllPaths   := TStringList.Create;
 fServerList := TStringList.Create;
 fClientList := TStringList.Create;
 fIAppServices := TMGIApplicationServicesStd.Create(self);
 MGDirectDllApplication := fIAppServices;
end;

destructor  TMGAppService.destroy;
begin
 DoOnDestroy;
{for i := 0 to DllList.Count-1 do
  freelibrary(DllList.Dlls[i].Handle);}
 DllList.free;
 fServerList.free;
 fClientList.free;
 fDllPaths.free;
 fIAppServices.Free;
 inherited;
end;

procedure TMGAppService.loaded;
begin
 inherited;
 DoOnCreate;
 DoRegister;
end;

procedure TMGAppService.SetDllPaths(value:TStrings);
begin
 fDllPaths.Assign(value);
end;

function  TMGAppService.GetProc(index:variant):TMGIProcServerIntf;
begin
 if VarType(index)=varString then ProcExists(index,result)
  else
  try
   if index>fServerList.Count-1 then result := nil else result := TMGIProcServerIntf(fServerList.Objects[index]);
  except
   result := nil;
  end;
end;

procedure TMGAppService.DoRegister;
var I,j : integer;
begin
 for i := 0 to screen.Formcount-1 do
  for j := 0 to screen.forms[i].Componentcount-1 do
   if (screen.forms[i].components[j] is TMGProcServer) then
   begin
    if (screen.forms[i].components[j] as TMGProcServer).Autoregister then (screen.forms[i].components[j] as TMGProcServer).register;
   end
   else
    if (screen.forms[i].components[j] is TMGProcClient) then
     if (screen.forms[i].components[j] as TMGProcClient).Autoregister then (screen.forms[i].components[j] as TMGProcClient).register;
end;

procedure TMGAppService.DoOnCreate;
begin
 if Assigned(fOnCreate) then OnCreate(self);
end;

procedure TMGAppService.DoOnDestroy;
begin
 if assigned(fOnDestroy) then OnDestroy(self);
end;

procedure TMGAppService.DoOnRegisterServer(proc:string);
begin
 if assigned(fOnRegServer) then OnRegisterServer(self,proc);
end;

procedure TMGAppService.DoOnRegisterClient(proc:string);
begin
 if assigned(fOnRegClient) then OnRegisterClient(self,proc);
end;

procedure TMGAppService.DoOnUnRegisterServer(proc:string);
begin
 if assigned(fOnUnRegServer) then OnUnRegisterServer(self,proc);
end;

procedure TMGAppService.DoOnUnRegisterClient(proc:string);
begin
 if assigned(fOnUnRegClient) then OnUnRegisterClient(self,proc);
end;

procedure TMGAppService.DoOnProcEnabled(Procname:string; isEnabled:boolean);
begin
 if assigned(fOnProcEnabled) then OnProcEnabledChange(self,ProcName,isEnabled);
end;

function  TMGAppService.DoExecuteProc(Proc:string; const Values: array of const):variant;
var temp : TMGIProcServerIntf;
begin
 if ProcExists(proc,temp) then result := temp.Execute(values)
  else result := null;
end;

procedure TMGAppService.DoRegisterProcServer(Proc:TMGIProcServerIntf);
begin
 if fServerlist.IndexOf(Proc.GetProcName)>-1 then raise EMGRegisterError.Create('Server can only be registered once!')
 else
 begin
  fServerList.AddObject(Proc.GetProcName,Proc);
  DoOnRegisterServer(Proc.GetProcName);
 end;
end;

procedure TMGAppService.DoRegisterProcClient(Proc:TMGIProcClientIntf);
begin
 fClientList.AddObject(Proc.GetProcName,Proc);
 DoOnRegisterClient(Proc.GetProcName);
end;

procedure TMGAppService.DoUnRegisterProcServer(Proc:string);
var I : integer;
begin
 i := fServerList.indexof(Proc);
 if i>-1 then
 begin
  fServerList.Delete(i);
  DoOnUnRegisterServer(Proc);
 end;
end;

procedure TMGAppService.DoUnRegisterProcClient(Proc:TMGIProcClientIntf);
var I : integer;
begin
 for i := 0 to fClientList.Count-1 do
  if fClientList.Objects[i]=Proc then
  begin
   fClientList.Delete(i);
   DoOnUnRegisterClient(proc.getprocname);
   break;
  end;
end;

procedure TMGAppService.DoNotifyEnabled(Proc:string; value:boolean);
var i:integer;
begin
 for i := 0 to fClientList.Count-1 do
  if TMGIProcClientIntf(fClientList.Objects[i]).GetProcName=Proc then
   TMGIProcClientIntf(fClientList.Objects[i]).OnEnabledChange(value);
end;

function  TMGAppService.DoGetProc(Proc:string):TMGIProcServerIntf;
begin
 ProcExists(Proc,Result);
end;

function  TMGAppService.DoIsProcEnabled(Proc:string):boolean;
var temp : TMGIProcServerIntf;
begin
 if ProcExists(Proc,temp) then result := temp.IsEnabled else result := false;
end;

procedure TMGAppService.NotifyClients(Proc,Event:string; value:variant);
var i:integer;
begin
 for i := 0 to fClientList.Count-1 do
  if TMGIProcClientIntf(fClientList.Objects[i]).GetProcName=Proc then
   TMGIProcClientIntf(fClientList.Objects[i]).OnServerEvent(event,value);
end;

procedure TMGAppService.UnRegisterdll(fdll : TMGIDllServicesIntf);
var id : string;
    t  : TMGDllItem;
begin
 id := fDll.GetIDString;
 t := DllList.Dlls[id];
 if assigned(t) then freelibrary(t.handle);
end;

function  TMGAppService.RegisterDll(DllPath:string):boolean;
var Handle : THandle;
    DllProc: TMGDllEntryPointProc;
    temp   : TMGIDllServicesIntf;
    i      : integer;
    t      : TMGDllItem;
begin
 if fileexists(dllpath) then
 begin
  handle := LoadLibrary(pchar(dllpath));
  try
   DllProc := TMGDllEntryPointProc(getprocAddress(handle,DllEntyPointName));
   if longint(@Dllproc)<>0 then Dllproc(fIAppServices,temp);
   if not DllList.Find(temp.GetIDString,i) then
   begin
    t := DllList.Add(temp.GetIDString);
    t.fFilename := dllpath;
    t.fHandle := Handle;
    t.fDllService := temp;
    temp.RegisterDll;
    result := true;
   end
   else
   begin
    result := false;
    freeLibrary(handle);
   end;
  except
   raise EMGDllRegisterError.Create('Error while registering DLL');
   result := false;
   freeLibrary(handle);
  end;
 end
 else result := false;
end;

function  TMGAppService.ProcExists(ProcName:string; var Proc:TMGIProcServerIntf):boolean;
var I : integer;
begin
 i := fServerList.IndexOf(Procname);
 if i>-1 then proc := TMGIProcServerIntf(fServerList.Objects[i]);
 result := i>-1;
end;

function  TMGAppService.CanCloseQuery:boolean;
var i : integer;
begin
 result := true;
 for i := 0 to fServerList.Count-1 do
  if TMGIProcServerIntf(fClientList.Objects[i]).CanClose = false then
  begin
   break;
   result := false;
  end;
end;

{*******************}


procedure Register;
begin
  RegisterComponents('MGSoft-DirectDll', [TMGProcServer, TMGProcClient, TMGDllService, TMGAppService]);
end;


procedure RegisterDll(IApp : TMGIApplicationServicesIntf; var IDll : TMGIDllServicesIntf);
begin
 if Assigned(MGDirectDllMainForm) then DllMainForm := MGDirectDllMainForm.Create(nil);
 MGDirectDllApplication := IApp;
 iDll := DllService;
end;

exports
  RegisterDll name DllEntyPointName;

initialization

finalization
 if Assigned(DllMainForm) then DllMainform.free;
end.

